import importlib.util
from .utils import input_or_output, get_asset_property
import logging
import importlib
# from alidadataset.serializations

def load(name, load_as="path"):
    
    module = importlib.import_module("alidadataset.serializations." + load_as)
    loading_func = getattr(module, "load")

    return loading_func(name)

def infer_module(name):
    storage = get_asset_property(name, "storage_type")
    print(storage)
    if storage == "filesystem":
        return "pandas_dataframe"
    elif storage == "kafka":
        if input_or_output(name) == "input":
            return "streaming_input"
        elif input_or_output(name) == "output":
            return "streaming_output"

def auto_load(name):
    return load(name=name, load_as=infer_module(name))


from ..utils import input_or_output, get_asset_property
from kafka import KafkaConsumer
import json


class Consumer(KafkaConsumer):
    def __init__(self, *args, **kwargs):
        super(Consumer, self).__init__(*args, **kwargs)

    def read_message(self):
        for element in self:
            yield json.loads(element.value.decode('utf-8'))
        

def load(name) -> Consumer:
    if get_asset_property(asset_name=name, property="storage_type") is not None:
        storage_type = get_asset_property(asset_name=name, property="storage_type")
    else:
        storage_type= "filesystem"
    
    
    if storage_type == "kafka":
        consumer = Consumer(
            get_asset_property(asset_name=name),
            bootstrap_servers=get_asset_property(asset_name=name, property="kafka_brokers").split(",")
        )
    
        return consumer



from ..utils import input_or_output, get_asset_property
from kafka import KafkaProducer
import json


class Producer(KafkaProducer):
    def __init__(self, *args, **kwargs):
        super(Producer, self).__init__(*args, **kwargs)

    def set_out_channel(self, channel):
        self.alida_out_channel = channel
        
    def send_message(self, value, *args, **kwargs):
        self.send(topic=self.alida_out_channel, value=json.dumps(value).encode('utf-8'), *args, **kwargs)


def load(name) -> Producer:
    if get_asset_property(asset_name=name, property="storage_type") is not None:
        storage_type = get_asset_property(asset_name=name, property="storage_type")
    else:
        storage_type= "filesystem"
    
    
    if storage_type == "kafka":
        producer = Producer(
            bootstrap_servers=get_asset_property(asset_name=name, property="kafka_brokers").split(",")
        )
        producer.set_out_channel(get_asset_property(asset_name=name))
        
        return producer



from datasets import Dataset
from .pandas_dataframe import load as load_df
import pyarrow as pa
from pyarrow import fs, csv, parquet
from minio import Minio as mn
from ..utils import input_or_output, get_asset_property

# def load(name)-> Dataset:
#     df = load_df(name=name)
#     return Dataset(arrow.Table.from_pandas(df))

def load_from_minio(name)-> Dataset:
    minio_url = get_asset_property(asset_name=name, property="minIO_URL")
    access_key = get_asset_property(asset_name=name, property="minIO_ACCESS_KEY")
    secret_key = get_asset_property(asset_name=name, property="minIO_SECRET_KEY")
    bucket_name = get_asset_property(asset_name=name, property="minio_bucket")
    

    use_ssl = get_asset_property(asset_name=name, property="use_ssl") if get_asset_property(asset_name=name, property="use_ssl") is not None else False
    use_ssl = True if use_ssl=="True" or use_ssl=="true" or use_ssl=="1" else False

    minio_client = mn(
        endpoint=minio_url if not ":" in minio_url else minio_url.split(":")[1][2:],
        access_key=access_key,
        secret_key=secret_key,
        secure=use_ssl 
    )

    minio = pa.fs.S3FileSystem(
        endpoint_override=minio_url,
        access_key=access_key,
        secret_key=secret_key,
        scheme="https" if use_ssl else "http")

    table = None

    folder_name = get_asset_property(asset_name=name) + "/"
    for obj in minio_client.list_objects(bucket_name, prefix=folder_name):
        csvFile = minio.open_input_file(bucket_name + "/" + obj.object_name)
        
        if table is None:
            table = pa.csv.read_csv(csvFile)
        else:
            table = pa.concat_tables([table, pa.csv.read_csv(csvFile)])

    return Dataset(table)


def load_from_disk(name)-> Dataset:
    df = load_df(name=name)
    return Dataset(pa.Table.from_pandas(df, preserve_index=False))


def load(name)-> Dataset:
    if get_asset_property(asset_name=name, property="storage_type") is not None:
        storage_type = get_asset_property(asset_name=name, property="storage_type")
    else:
        storage_type= "filesystem"

    if storage_type == "minio":
        return load_from_minio(name)
    elif storage_type =="filesystem":
        return load_from_disk(name)



from ..utils import input_or_output, get_asset_property
from dsioutilities import Dataset as alidaDataset
from datasets import Dataset
from datasets import load_from_disk
from s3fs import S3FileSystem


def load_from_minio(name):
    
    minio_url = get_asset_property(asset_name=name, property="minIO_URL")
    access_key = get_asset_property(asset_name=name, property="minIO_ACCESS_KEY")
    secret_key = get_asset_property(asset_name=name, property="minIO_SECRET_KEY")
    bucket_name = get_asset_property(asset_name=name, property="minio_bucket")

    use_ssl = get_asset_property(asset_name=name, property="use_ssl") if get_asset_property(asset_name=name, property="use_ssl") is not None else False
    use_ssl = True if use_ssl=="True" or use_ssl=="true" or use_ssl=="1" else False

    s3 = S3FileSystem(
        key=access_key, 
        secret=secret_key,
        client_kwargs={'endpoint_url': minio_url},
        use_ssl=use_ssl
    )
    
    return load_from_disk("s3://" + bucket_name + "/" + get_asset_property(name), storage_options=s3.storage_options)


def load(name)-> Dataset:
    if get_asset_property(asset_name=name, property="storage_type") is not None:
        storage_type = get_asset_property(asset_name=name, property="storage_type")
    else:
        storage_type= "filesystem"

    if storage_type == "minio":
        return load_from_minio(name)
    elif storage_type =="filesystem":
        return load_from_disk(get_asset_property(asset_name=name))



from ..utils import input_or_output, get_asset_property
from kafka import KafkaConsumer, KafkaProducer
import typing



def load(name):
    if get_asset_property(asset_name=name, property="storage_type") is not None:
        storage_type = get_asset_property(asset_name=name, property="storage_type")
    else:
        storage_type= "filesystem"
    
    
    if storage_type == "kafka":
            
        if input_or_output(name=name) == "input":
            consumer = KafkaConsumer(
                get_asset_property(asset_name=name), 
                bootstrap_servers=get_asset_property(asset_name=name, property="kafka_brokers").split(",")
            )
            return typing.cast(KafkaConsumer, consumer)

        elif input_or_output(name=name) == "output":
            producer = KafkaProducer(
                bootstrap_servers=get_asset_property(asset_name=name, property="kafka_brokers").split(",")
            )
            return typing.cast(KafkaProducer, producer)



from alidaargparser import get_asset_property
from importlib.util import find_spec

def input_or_output(name):

    dir = get_asset_property(name, property="direction")
    
    # TODO remove if else when multidataset supported
    if dir is not None:
        return dir
    else:
        if name == "input_dataset" or name == "input-dataset":
            return "input"
        elif name == "output_dataset" or name == "output-dataset":
            return "output"
        else:
            return None


def packages_are_installed(packages):
    for package in packages:
        if find_spec(package) is None:
            return False
    return True



from pandas import DataFrame
from pandas import read_csv
import pandas as pd
from ..utils import input_or_output, get_asset_property
from dsioutilities import Dataset
import os

def load_dataframe_from_minio(name):
    input_dataset = Dataset(name, dataset_type="tabular")
    if isinstance(input_dataset.get_path(), list):
        df = pd.concat([pd.read_csv(filename) for filename in input_dataset.get_path()])
    else:
        df = pd.read_csv(input_dataset.get_path())
    return df


def load(name)-> DataFrame:
    if get_asset_property(asset_name=name, property="storage_type") is not None:
        storage_type = get_asset_property(asset_name=name, property="storage_type")
    else:
        storage_type= "filesystem"
    
    if storage_type == "minio":
        return load_dataframe_from_minio(name)
    elif storage_type =="filesystem":
        if os.path.isdir(get_asset_property(asset_name=name)):
            list_of_files = next(os.walk(get_asset_property(asset_name=name)), (None, None, []))[2]
            return pd.concat([pd.read_csv(os.path.join(get_asset_property(asset_name=name), filename)) for filename in list_of_files if filename.endswith(".csv")])
        else:
            return read_csv(get_asset_property(asset_name=name))
